from sqlalchemy import CHAR
from sqlalchemy import DATE
from sqlalchemy import JSON
from sqlalchemy import ARRAY
from sqlalchemy import Column
from sqlalchemy import Boolean
from sqlalchemy import Numeric
from sqlalchemy import Integer
from sqlalchemy import VARCHAR
from sqlalchemy import DateTime
from sqlalchemy import Sequence
from sqlalchemy import ForeignKey
from sqlalchemy.sql import func
from sqlalchemy.orm import relationship

from asc.core import env
from asc.core.database.model import Model
from asc.models.melhorenvio import OrderDelivery


class Order(Model):
    __tablename__ = 'order'

    id_order = Column(Integer, primary_key=True)
    id_user = Column(Integer, nullable=False)
    id_order_status = Column(Integer, nullable=False, server_default='7')
    id_shipment = Column(Integer, nullable=False)
    ticket = Column(VARCHAR, nullable=False)
    id_order_delivery = Column(ForeignKey('private.order_delivery.id_order_delivery', ondelete='CASCADE', onupdate='RESTRICT'))

    order_id_order_delivery_fkey = relationship('OrderDelivery')


class Payment(Model):
    __tablename__ = 'marketplace_payment'

    id_marketplace_payment = Column(Integer, primary_key=True)
    id_marketplace_external = Column(VARCHAR, nullable=False)
    id_marketplace_client = Column(Integer)
    id_storage = Column(Integer)
    dt_add = Column(DATE, server_default=func.now())
    status = Column(VARCHAR, server_default='started')
    paid = Column(Boolean, server_default='false')
    gateway = Column(VARCHAR)
    transaction_method = Column(VARCHAR)
    transaction_amount = Column(Integer)
    taxes_amount = Column(Integer)
    cd_reference = Column(Integer)
    meta = Column(JSON)


class Tax(Model):
    """ Configurações de taxas a serem utilizadas
    por drivers de pagamento. """
    __tablename__ = 'marketplace_tax'

    id_marketplace_tax = Column(Integer, primary_key=True)
    id_marketplace_payment_method = Column(Integer)
    id_marketplace_payment_gateway = Column(Integer)
    dt_add = Column(DateTime, server_default=func.now())
    cust_process_type = Column(Integer)
    cust_process_amount = Column(Numeric(4,2))
    cust_split_type = Column(Integer)
    cust_split_amount = Column(ARRAY(Numeric(4,2)))
    is_active = Column(Boolean, server_default='false')


class Client(Model):
    __tablename__ = 'marketplace_client'

    id_marketplace_client = Column(Integer, primary_key=True)
    name = Column(VARCHAR, nullable=False)
    email = Column(VARCHAR, nullable=False)
    phone = Column(VARCHAR, nullable=False)
    document = Column(VARCHAR, nullable=False)


class User(Model):
    __tablename__ = 'user'

    id_user = Column(Integer, primary_key=True)
    name = Column(VARCHAR)
    udoc = Column(VARCHAR)
    utel = Column(VARCHAR)
    mail = Column(VARCHAR)


class UserCard(Model):
    __tablename__ = 'user_card'

    id_user_card = Column(Integer,primary_key=True)
    id_user = Column(ForeignKey('private.user.id_user'),nullable=False)
    dt_add = Column(DATE,nullable=False)
    dt_exp = Column(CHAR(4),nullable=False)
    nm_holder = Column(VARCHAR,nullable=False)
    nr_card = Column(CHAR(64),nullable=False)
    nr_cvv = Column(CHAR(3),nullable=False)
    id_user_endereco = Column(Integer,nullable=False)
    hs_card = Column(VARCHAR)

