import codecs
import smtplib
from os import path
from email import encoders
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication

from asc.core import env, log


class Mailer:
    """ Classe responsável por envio de emails. """
    attachs = []
    attachsType = 'text'

    def __init__(self, **kwargs):
        """Primeiras configurações"""
        self.mailHost = kwargs.get('host', env.get('mailer_host'))
        self.mailFrom = kwargs.get('mailFrom', env.get('mailer_from'))
        self.mailPass = kwargs.get('mailPass', env.get('mailer_pass'))
        self.mailPort = kwargs.get('port', env.get('mailer_port'))

        self.mailBody = MIMEMultipart()
        self.serv: smtplib.SMTP = smtplib.SMTP(self.mailHost, self.mailPort)
        try:
            self.serv.starttls()

            # self.serv.ehlo()
            # self.serv.starttls()
            self.serv.login(self.mailFrom, self.mailPass)
        except Exception as e:
            log.error(e)

    def send_mail(self, **kwargs):
        """
            Envia email
        """

        self.mailTo = kwargs.get('to')
        self.mailHtml = kwargs.get('html')
        self.mailName = kwargs.get('subject')

        self.mailBody.attach(MIMEText(self.mailHtml, 'html'))
        self.mailBody['To'] = self.mailTo
        self.mailBody['From'] = self.mailFrom
        self.mailBody['Subject'] = self.mailName

        for attach in self.attachs:

            file = attach

            if type(attach) == list:
                name = attach[0]
                file = attach[1]

            isFile = os.path.isfile(file)

            if self.attachsType == 'text':
                self.mailBody.attach(MIMEText(open(file).read()))

            if self.attachsType == 'application':
                binary_pdf = open(file, 'rb')

                payload = MIMEBase('application', 'octate-stream', Name=name)
                payload.set_payload((binary_pdf).read())

                # enconding the binary into base64
                encoders.encode_base64(payload)

                # add header with pdf name
                payload.add_header('Content-Decomposition', 'attachment',
                                   Name=name)
                self.mailBody.attach(payload)

        return self.serv.sendmail(self.mailFrom, self.mailTo,
                               self.mailBody.as_string())

    def send_mail_slug(self, slug:str, to:str, subject:str, html_vars: dict = {}, attach=[]):

        self.attachs = attach

        if not slug:
            raise Exception('Código HTML não encontrado')

        file = codecs.open(f'{env.get("mailer_html")}'
                               f'email-{slug}.html', 'r')

        html = file.read()
        file.close()

        for k, v in html_vars.items():
            if isinstance(v, bool):
                continue

            html = html.replace('{{%s}}' % k, v)

        send = self.send_mail(
            to=to,
            subject=subject,
            html=html
        )

        return send

    def __del__(self):
        try:
            self.serv.quit()
        except:
            pass
